<?php

/*
  Plugin Name: GS GoMobi Redirector
  Plugin URI: http://www.geekstuff.co.nz/2012/12/29/gs-gomobi-redirector-plugin-for-wordpress/
  Description: This plugin integrates a goMobi mobile website with your full-size Wordpress website. Mobile visitors are detected and automatically redirected to your goMobi mobile website. The plugin incorporates the official PHP redirection code generated by the goMobi service. It also incorporates some patches by Nick Rice <a href="http://bit.ly/YLBvb0" target="_blank">documented in this forum post</a>
  Version: 1.1
  Author: GeekStuff Ltd
  Author URI: http://www.geekstuff.co.nz
  License: GPL2
*/


// BEGIN ********* Settings Page
add_action( 'admin_menu', 'my_admin_menu' );
function my_admin_menu() {
    add_options_page( 'GS GoMobi Redirector', 'GS GoMobi Redirector', 'manage_options', 'my-plugin', 'my_options_page' );
}

add_action( 'admin_init', 'my_admin_init' );
function my_admin_init() {
    register_setting( 'gmr-settings-group', 'gsgmr-enabled');
    register_setting( 'gmr-settings-group', 'gsgmr-rootonly');
    register_setting( 'gmr-settings-group', 'gsgmr-url', 'gsgmr_url_validate' );
    register_setting( 'gmr-settings-group', 'gsgmr-cookie-timeout' );
    register_setting( 'gmr-settings-group', 'gsgmr-tablet-url' );
    register_setting( 'gmr-settings-group', 'gsgmr-hook-priority' );
    add_settings_section( 'section-one', 'goMobi Site Details', 'section_one_callback', 'gmr-plugin' );
    add_settings_field( 'gsgmr-enabled', 'Enable redirection', 'gmr_enabled_callback', 'gmr-plugin', 'section-one' );
    add_settings_field( 'gsgmr-url', 'goMobi site URL', 'gmr_url_callback', 'gmr-plugin', 'section-one' );

    add_settings_section( 'section-two', '<hr />Advanced Settings', 'section_two_callback', 'gmr-plugin' );
    add_settings_field( 'gsgmr-cookie-timeout', 'Cookie Timeout (secs)', 'gmr_timeout_callback', 'gmr-plugin', 'section-two' );
    add_settings_field( 'gsgmr-tablet-url', 'Tablet URL', 'gmr_tablet_callback', 'gmr-plugin', 'section-two' );
    add_settings_field( 'gsgmr-hook-priority', 'Hook Priority', 'gmr_hook_callback', 'gmr-plugin', 'section-two' );
    add_settings_field( 'gsgmr-rootonly', 'Redirect Homepage only', 'gmr_rootonly_callback', 'gmr-plugin', 'section-two' );

}

function section_one_callback() {
    echo 'Enter your goMobi mobile site URL in the field below. Use the Enable checkbox to enable/disable mobile redirection.';
}

function section_two_callback() {
    echo 'You should only modify these settings if you are confident that you understand their effect<br />
        <strong>Tablet URL</strong> allows you to enter a different target URL for Tablet devices if required.<br />
        <strong>Hook priority</strong> affects the order in which this plugin will be called relative to other plugins. Default is 10.<br />
        As redirect headers must be processed before any other output it is possible that other plugins that use the WP init action may prevent the redirect.<br />
        In that case, adjusting the hook priority to a lower value <em>may</em> help. Your mileage may vary.<br /><br />
        <strong>Redirect Homepage</strong> only is set to YES by default. Turning this off will result in mobile users being redirected to your goMobi site if they directly access a sub-page of your site.<br />
        In addition, when you try and access the Admin Dashboard from a mobile device you will be redirected. Both of these behaviours are undesirable so you should leave this setting enabled';
}

function gmr_enabled_callback() {
    if(get_option( 'gsgmr-enabled') == 1) {
        $setting="checked='checked'";
    }else {
        $setting= "";
    }
    echo "<input type='checkbox' name='gsgmr-enabled' value='1' $setting />";
}
function gmr_url_callback() {
    $setting = esc_attr( get_option( 'gsgmr-url' ) );
    echo "<input type='text' name='gsgmr-url' value='$setting' />";
}
function gmr_timeout_callback() {
    $setting = esc_attr( get_option( 'gsgmr-cookie-timeout' ) );
    $dsetting = $setting ? $setting : 900;
    echo "<input type='text' name='gsgmr-cookie-timeout' value='$dsetting' />";
}

function gmr_tablet_callback() {
    $setting = esc_attr( get_option( 'gsgmr-tablet-url' ) );
    echo "<input type='text' name='gsgmr-tablet-url' value='$setting' />";
}

function gmr_hook_callback() {
    $setting = esc_attr( get_option( 'gsgmr-hook-priority' ) );
    $o1 = $o2 = $o3 = "";
    if($setting == 10)
        $o1 = "selected = 'selected'";
    elseif($setting == 5)
        $o2 = "selected = 'selected'";
    elseif($setting == 0)
        $o3 = "selected = 'selected'";

    echo "<select name='gsgmr-hook-priority' value='$setting'><option $o1>10</option><option $o2>5</option><option $o3>0</option></select>";
}

function gmr_rootonly_callback() {
    $ronly = get_option( 'gsgmr-rootonly');
    if($ronly === FALSE) {
        $setting="checked='checked'"; // new setting added - default to checked  
        add_option('gsgmr-rootonly','1');   // and add it to the DB safely
    }elseif($ronly == 1) {
        $setting="checked='checked'";
    }elseif($ronly == 1) {
        $setting= "";
    }
    echo "<input type='checkbox' name='gsgmr-rootonly' value='1' $setting />";
}

function gsgmr_url_validate( $input ) {
    $output = get_option( 'gsgmr-url' );

    if ( validateURL( $input ) )
        $output = $input;
    else
        add_settings_error( 'gsgmr-url', 'invalid-gomobi-url', 'Not Saved - You  entered an invalid goMobi site URL. Must be in the format http://your.gomobi.site' );

    return $output;
}
function gsgmr_tablet_validate( $input ) {
    $output = get_option( 'gsgmr-tablet-url' );

    if ( validateURL( $input ) )
        $output = $input;
    else
        add_settings_error( 'gsgmr-tablet-url', 'invalid-tablet-url', 'Not Saved - You  entered an invalid Tablet URL. Must be in the format http://your.gomobi.site' );

    return $output;
}

function my_options_page() {
    $pdata = get_plugin_data(__FILE__ );
    ?>
<div class="wrap">
    <h2>GS GoMobi Redirector Settings</h2>
    <form action="options.php" method="POST">
            <?php settings_fields( 'gmr-settings-group' ); ?>
            <?php do_settings_sections( 'gmr-plugin' ); ?>
            <?php submit_button(); ?>
    </form>
    <span style="font-size:small;font-style:italic;">plugin version <?php echo($pdata['Version']); ?></span>
</div>
    <?php
}

// END *********** Settings Page

function validateURL($URL) {
    $v = "/^(http|https|ftp):\/\/([A-Z0-9][A-Z0-9_-]*(?:\.[A-Z0-9][A-Z0-9_-]*)+):?(\d+)?\/?/i";
    return (bool)preg_match($v, $URL);
}


$pty = esc_attr( get_option( 'gsgmr-hook-priority' ) );
$priority = $pty ? $pty : 10;   // default to 10
add_action('init', 'gmr_redirect', $priority);

function gmr_redirect() {
// we only want to redirect from the site homepage
// direct calls to sub-pages or the admin area will not be redirected
    $rootonly = 1;
    $rootonly = get_option( 'gsgmr-rootonly' );
    $theuri = $_SERVER["REQUEST_URI"];    
    if($rootonly == 1 && $theuri != '/') {
        return;
    }
    // get the goMobi site URL from the wp_options table
    // if blank or invalid then just return
    $gomobiurl = get_option( 'gsgmr-url' );
    $cookietimeout = get_option( 'gsgmr-cookie-timeout' );
    $tableturl = get_option( 'gsgmr-tablet-url' );
    $enabled = get_option( 'gsgmr-enabled' );
    if(!$enabled || empty($gomobiurl) || !validateURL($gomobiurl)) {
        return;
    }

    define('MOBILE_SITE', $gomobiurl);
## Uncomment the line below to redirect to site specifically for tablets
    define('DA_USE_COOKIES', true);
    define('DA_USE_CACHE', true);
    define('COOKIE_EXPIRY_TIME', 900);
    define('DA_CACHE_DIR', sys_get_temp_dir() . DIRECTORY_SEPARATOR . 'DeviceAtlasCache' . DIRECTORY_SEPARATOR);
    define('DA_URI', 'http://detect.deviceatlas.com/query?User-Agent=%s');

    $redirect = isset($_REQUEST['redirect']) ? $_REQUEST['redirect'] : null;

    if ($redirect != 'false') {
        $da_results = array('_source' => 'none');

        $prevent_redirect = false;
        if (DA_USE_COOKIES && isset($_COOKIE['Mobi_Mtld_DA_Prevent_Redirect'])) {

            //Clear the redirect-prevention cookie if we see redirect=true parameter
            if ($redirect == 'true') {
                setcookie('Mobi_Mtld_DA_Prevent_Redirect', 'false', time() - 3600);
            } else {
                $prevent_redirect = isset($_COOKIE['Mobi_Mtld_DA_Prevent_Redirect']);
                if ($prevent_redirect) {
                    $da_results['_source'] = 'prevent_redirect';
                }
            }
        }


        if (DA_USE_COOKIES && isset($_COOKIE['Mobi_Mtld_DA_Properties'])) {
            $props_cookie = $_COOKIE['Mobi_Mtld_DA_Properties'];
            $props_cookie = str_replace('\\', '', $props_cookie);
            $da_results = (array) json_decode($props_cookie, true);
            $da_results['_source'] = 'cookie';
        }

        if (DA_USE_CACHE && $da_results['_source'] === 'none') {
            $da_cache_file = md5($_SERVER["HTTP_USER_AGENT"]) . '.json';

            if (!file_exists(DA_CACHE_DIR) && !@mkdir(DA_CACHE_DIR)) {
                $da_results['_error'] = "Unable to create cache directory: " . DA_CACHE_DIR . "\n";
            } else {
                $da_json = @file_get_contents(DA_CACHE_DIR . $da_cache_file);

                if ($da_json !== false) {
                    $da_results = (array) json_decode($da_json, true);
                    $da_results['_source'] = 'cache';

                    if (DA_USE_COOKIES) {
                        setcookie('Mobi_Mtld_DA_Properties', $da_json);
                    }
                }
            }
        }

        if ($da_results['_source'] === 'none') {
            $url = sprintf(DA_URI, urlencode($_SERVER["HTTP_USER_AGENT"]));

            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            $da_json = curl_exec($ch);
            curl_close($ch);

            if ($da_json !== false) {
                $da_results = array_merge(json_decode($da_json, true), $da_results);
                $da_results['_source'] = 'webservice';

                if (DA_USE_COOKIES) {
                    setcookie('Mobi_Mtld_DA_Properties', $da_json);
                }

                if (DA_USE_CACHE) {
                    if (@file_put_contents(DA_CACHE_DIR . $da_cache_file, $da_json) === false) {
                        $da_results['_error'] .= "Unable to write cache file " . DA_CACHE_DIR . $da_cache_file . "\n";
                    }
                }
            } else {
                $da_results['_error'] .= "Error fetching DeviceAtlas data from webservice.\n";
            }
        }

        if (!$prevent_redirect) {
            if (!empty($tableturl) && isset($da_results['isTablet']) && $da_results['isTablet'] == 'true') {
                $redirect_url = $tableturl;
            } else if (isset($da_results['mobileDevice']) && $da_results['mobileDevice'] == 'true') {
                $redirect_url = $gomobiurl;
            }

            if (isset($redirect_url)) {
                wp_redirect($redirect_url);
                exit();
            }
        }
    } else {
        //Set a cookie so we remember not to redirect again in the future
        if (DA_USE_COOKIES && !isset($_COOKIE['Mobi_Mtld_DA_Prevent_Redirect'])) {
            setcookie('Mobi_Mtld_DA_Prevent_Redirect', "true", time() + COOKIE_EXPIRY_TIME);
        }
    }
}

?>
